@@ -2,16 +2,18 @@ import os
 from pathlib import Path
 import yaml
 
-from steam_sdk.data.DataModel import *
-from steam_sdk.data.DataCircuit import DataCircuit
+from steam_sdk.builders.BuilderPyBBQ import BuilderPyBBQ
+from steam_sdk.data.DataModelConductor import DataModelConductor
+from steam_sdk.data.DataModelMagnet import *
+from steam_sdk.data.DataModelCircuit import DataModelCircuit
 from steam_sdk.data.DataRoxieParser import APIdata
 from steam_sdk.builders.BuilderLEDET import BuilderLEDET
 from steam_sdk.builders.BuilderSIGMA import BuilderSIGMA
 from steam_sdk.builders.BuilderProteCCT import BuilderProteCCT
-from steam_sdk.builders.BuilderCircuit import BuilderCircuit
 from steam_sdk.parsers.ParserLEDET import ParserLEDET, copy_modified_map2d_ribbon_cable, copy_map2d
 from steam_sdk.parsers.ParserPSPICE import ParserPSPICE
 from steam_sdk.parsers.ParserProteCCT import ParserProteCCT
+from steam_sdk.parsers.ParserProtePyBBQ import ParserPyBBQ
 from steam_sdk.parsers.ParserRoxie import ParserRoxie
 from steam_sdk.parsers.ParserMap2d import getParametersFromMap2d
 from steam_sdk.plotters import PlotterRoxie
@@ -39,7 +41,7 @@ class BuilderModel:
         # Unpack arguments
         self.software: List[str] = software
         if not self.software: self.software = []  # This avoids error when None is passed to software # TODO: Verify this is sound coding
-        valid_cases = ['magnet', 'circuit']
+        valid_cases = ['magnet', 'circuit', 'conductor']
         if case_model in valid_cases:
             self.case_model = case_model
         else:
@@ -65,7 +67,7 @@ class BuilderModel:
         ### Case of a magnet model
         if case_model == 'magnet':
             # Initialize
-            self.model_data: DataModel = DataModel()
+            self.model_data: DataModelMagnet = DataModelMagnet()
             self.roxie_data: APIdata = APIdata()
             self.path_magnet = None
             self.path_data = None
@@ -99,12 +101,12 @@ class BuilderModel:
 
                 if flag_plot_all:
                     PlotterRoxie.plot_all(self.roxie_data)
-                    PlotterModel.plot_all(self.model_data)
+                    PlotterModel.plot_all(self.model_data, self.roxie_data)
 
         ### Case of a circuit model
         elif case_model == 'circuit':
             # Initialize
-            self.circuit_data: DataCircuit = DataCircuit()
+            self.circuit_data: DataModelCircuit = DataModelCircuit()
             self.path_settings = None
 
             # If flag_build set true, the model will be generated during the initialization
@@ -119,12 +121,30 @@ class BuilderModel:
                 self.set_input_paths(self.case_model)
 
                 # Build circuit model
-                self.buildCircuit()
-
-                # Write circuit netlist
                 for s in self.software:
                     if s == 'PSPICE': self.buildPSPICE()
 
+        ### Case of a conductor model
+        elif case_model == 'conductor':
+            # Initialize
+            self.conductor_data: DataModelConductor = DataModelConductor()
+            self.path_settings = None
+
+            # If flag_build set true, the model will be generated during the initialization
+            if flag_build:
+                if self.verbose:
+                    print('output_path: {}'.format(self.output_path))
+
+                # Load model data from the input .yaml file
+                self.loadModelData(self.case_model)
+
+                # Set paths of input files and settings
+                self.set_input_paths(self.case_model)
+
+                # Build model
+                for s in self.software:
+                    if s == 'PyBBQ':    self.buildPyBBQ()
+
 
     def set_input_paths(self, case_model: str):
         """
@@ -147,7 +167,7 @@ class BuilderModel:
                 self.path_cadata = Path(self.path_magnet / self.model_data.Sources.conductor_fromROXIE).resolve()
             if self.model_data.Sources.iron_fromROXIE:
                 self.path_iron = Path(self.path_magnet / self.model_data.Sources.iron_fromROXIE).resolve()
-        elif case_model == 'circuit':
+        elif case_model in ['circuit', 'conductor']:
             self.path_data   = None
             self.path_map2d  = None
             self.path_cadata = None
@@ -161,6 +181,8 @@ class BuilderModel:
         for environ_var in ['HOMEPATH', 'SWAN_HOME']:
             if environ_var in os.environ:
                 user_name = os.path.basename(os.path.normpath(os.environ[environ_var]))
+        if self.verbose:
+            print('user_name:   {}'.format(user_name))
 
         # TODO: Change this logic with a tree structure depending on the current location, not the file location
         if self.relative_path_settings == '':
@@ -172,7 +194,7 @@ class BuilderModel:
             with open(Path.joinpath(self.path_settings, f"settings.{user_name}.yaml"), 'r') as stream:
                 self.settings_dict = yaml.safe_load(stream)
         else:
-            with open(Path.joinpath(Path(os.getcwd()).parent, "settings.user.yaml"), 'r') as stream:
+            with open(Path.joinpath(Path(os.getcwd()).parent, "settings.SYSTEM.yaml"), 'r') as stream:
                 self.settings_dict = yaml.safe_load(stream)
             # raise Exception('Cannot find paths without settings file')
 
@@ -197,15 +219,20 @@ class BuilderModel:
             raise Exception('No .yaml path provided.')
 
         if case_model == 'magnet':
-            # Load yaml keys into DataModel dataclass
+            # Load yaml keys into DataModelMagnet dataclass
             with open(self.file_model_data, "r") as stream:
                 dictionary_yaml = yaml.safe_load(stream)
-                self.model_data = DataModel(**dictionary_yaml)
+                self.model_data = DataModelMagnet(**dictionary_yaml)
         elif case_model == 'circuit':
-            # Load yaml keys into DataCircuit dataclass
+            # Load yaml keys into DataModelCircuit dataclass
             with open(self.file_model_data, "r") as stream:
                 dictionary_yaml = yaml.safe_load(stream)
-                self.circuit_data = DataCircuit(**dictionary_yaml)
+                self.circuit_data = DataModelCircuit(**dictionary_yaml)
+        elif case_model == 'conductor':
+            # Load yaml keys into DataModelConductor dataclass
+            with open(self.file_model_data, "r") as stream:
+                dictionary_yaml = yaml.safe_load(stream)
+                self.conductor_data = DataModelConductor(**dictionary_yaml)
 
 
     def loadRoxieData(self):
@@ -238,8 +265,7 @@ class BuilderModel:
             Building a LEDET model
         """
         magnet_name = self.model_data.GeneralParameters.magnet_name
-        nameFileSMIC = os.path.join(self.output_path,
-                                    magnet_name + '_selfMutualInductanceMatrix.csv')  # full path of the .csv file with self-mutual inductances to write
+        nameFileSMIC = os.path.join(self.output_path, magnet_name + '_selfMutualInductanceMatrix.csv')  # full path of the .csv file with self-mutual inductances to write
 
         # Copy/edit the ROXIE map2d file
         suffix = "_All"
@@ -247,30 +273,37 @@ class BuilderModel:
             #     # [[...half_turn_length, Ribbon...n_strands],.....]
             # TODO: geometry when conductor has a combination of ribbon and non-ribbon cables
 
-            nT_from_original_map2d, _, _, _, _, _, _, _ = getParametersFromMap2d(
+            # List of flags that are True is the cable type is "Ribbon"
+            list_flag_ribbon = []
+            for i, cond in enumerate(self.model_data.CoilWindings.conductor_to_group):
+                list_flag_ribbon.append(self.model_data.Conductors[cond-1].cable.type == 'Ribbon')
+
+            nT_from_original_map2d, nStrands_inGroup_original_map2d, _, _, _, _, _, _ = getParametersFromMap2d(
                 map2dFile=self.path_map2d, headerLines=self.model_data.Options_LEDET.field_map_files.headerLines,
                 verbose=self.verbose)
 
             n_groups_original_file = len(nT_from_original_map2d)
-            geometry_ribbon_cable = [[None, None]] * n_groups_original_file
+            geometry_ribbon_cable = []
 
             for i in range(n_groups_original_file):
-                geometry_ribbon_cable[i][0] = self.model_data.Conductors[
-                    self.model_data.CoilWindings.conductor_to_group[i] - 1].cable.n_strands  # layers
-                geometry_ribbon_cable[i][1] = nT_from_original_map2d[i]  # number of half-turns
+                list = [None, None]
+                list[0] = int(nStrands_inGroup_original_map2d[i])  # layers
+                list[1] = nT_from_original_map2d[i]  # number of half-turns; in case it is not a ribbon cable, it is going to be ignored in the modify-ribbon-cable function
+                geometry_ribbon_cable.append(list)
 
             if self.verbose:
                 print('geometry_ribbon_cable: {}'.format(geometry_ribbon_cable))
 
-            file_name_output = copy_modified_map2d_ribbon_cable(self.model_data.GeneralParameters.magnet_name,
+            file_name_output = copy_modified_map2d_ribbon_cable(magnet_name,
                                                                 self.path_map2d,
                                                                 self.output_path, geometry_ribbon_cable,
                                                                 self.model_data.Options_LEDET.field_map_files.flagIron,
                                                                 self.model_data.Options_LEDET.field_map_files.flagSelfField,
+                                                                list_flag_ribbon,
                                                                 suffix=suffix, verbose=self.verbose)
 
         elif self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == 0 or self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == None:
-            file_name_output = copy_map2d(self.model_data.GeneralParameters.magnet_name, self.path_map2d,
+            file_name_output = copy_map2d(magnet_name, self.path_map2d,
                                           self.output_path, self.model_data.Options_LEDET.field_map_files.flagIron,
                                           self.model_data.Options_LEDET.field_map_files.flagSelfField, suffix=suffix,
                                           verbose=self.verbose)
@@ -279,7 +312,27 @@ class BuilderModel:
 
         builder_ledet = BuilderLEDET(path_magnet=self.path_magnet, input_model_data=self.model_data,
                                      input_roxie_data=self.roxie_data, input_map2d=self.map2d_file_modified,
-                                     smic_write_path=nameFileSMIC, flag_build=self.flag_build, verbose=self.verbose)
+                                     smic_write_path=nameFileSMIC, flag_build=self.flag_build, flag_plot_all=self.flag_plot_all,
+                                     verbose=self.verbose)
+
+        # Copy or modify+copy magnet-name_E....map2d files
+        number_input_files = len([entry for entry in os.listdir(self.path_magnet) if os.path.isfile(os.path.join(self.path_magnet, entry))])
+        for file in range(number_input_files+1):
+            suffix = '_E{}'.format(file)
+            path_map2d_E = os.path.join(self.path_magnet, magnet_name + suffix + '.map2d')
+            if os.path.isfile(path_map2d_E):
+                if self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == 1:
+                    copy_modified_map2d_ribbon_cable(magnet_name,
+                                                     path_map2d_E,
+                                                     self.output_path, geometry_ribbon_cable,
+                                                     self.model_data.Options_LEDET.field_map_files.flagIron,
+                                                     self.model_data.Options_LEDET.field_map_files.flagSelfField,
+                                                     list_flag_ribbon,
+                                                     suffix=suffix, verbose=self.verbose)
+                elif self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == 0 or self.model_data.Options_LEDET.field_map_files.flag_modify_map2d_ribbon_cable == None:
+                    copy_map2d(magnet_name, path_map2d_E, self.output_path, self.model_data.Options_LEDET.field_map_files.flagIron,
+                                          self.model_data.Options_LEDET.field_map_files.flagSelfField, suffix=suffix,
+                                          verbose=self.verbose)
 
         # Write output excel file
         parser_ledet = ParserLEDET(builder_ledet)
@@ -313,26 +366,32 @@ class BuilderModel:
                      settings_dict=self.settings_dict, output_path=self.output_path,
                      flag_build=self.flag_build, verbose=self.verbose)
 
-
-    def buildCircuit(self):
-        """
-            Build a circuit netlist model
-        """
-        self.builder_Circuit = BuilderCircuit(path_parent=self.path_magnet, circuit_data=self.circuit_data,
-                                              flag_build=self.flag_build, verbose=self.verbose)
-
     def buildPSPICE(self):
         """
             Build a PSPICE circuit netlist model
         """
+        circuit_data = self.circuit_data
         circuit_name = self.circuit_data.GeneralParameters.circuit_name
-        circuit_data = self.builder_Circuit.circuit_data
 
-        # Write output excel file
-        parser_pspice = ParserPSPICE(circuit_data=circuit_data)
+        # Write output .cir file
+        parser_pspice = ParserPSPICE(circuit_data=circuit_data, path_input=self.path_magnet, output_path=self.output_path)
         nameFilePSPICE = os.path.join(self.output_path, circuit_name + '.cir')  # full path of the PSPICE netlist to write
         parser_pspice.write2pspice(full_path_file_name=nameFilePSPICE, verbose=self.verbose)
 
+        # Copy additional files
+        parser_pspice.copy_additional_files()
+
+    def buildPyBBQ(self):
+        """
+            Building a PyBBQ model
+        """
+        conductor_name = self.conductor_data.GeneralParameters.conductor_name
+        builder_PyBBQ = BuilderPyBBQ(input_model_data=self.conductor_data, flag_build=self.flag_build, verbose=self.verbose)
+
+        # Write output yaml file
+        parser_PyBBQ = ParserPyBBQ(builder_PyBBQ)
+        nameFilePyBBQ = os.path.join(self.output_path, conductor_name + '.yaml')  # full path of the PyBBQ input file to write
+        parser_PyBBQ.writePyBBQ2yaml(full_path_file_name=nameFilePyBBQ, verbose=self.verbose)
 
     def dumpAll(self):
         """
@@ -364,26 +423,4 @@ class BuilderModel:
         with open(dump_all_full_path, 'w') as outfile:
             yaml.dump(all_data_dict, outfile, default_flow_style=False, sort_keys=False)
 
-    def get_conductor_corners(self):
-        ''' Returns conductor corner positions (insulated and bare)  and conductor currents in list form '''
-        xInsuPos = []
-        yInsuPos = []
-        xBarePos = []
-        yBarePos = []
 
-        iConductor = []
-        for coil_nr, coil in self.roxie_data.coil.coils.items():
-            for pole_nr, pole in coil.poles.items():
-                for layer_nr, layer in pole.layers.items():
-                    for winding_key, winding in layer.windings.items():
-                        for block_key, block in winding.blocks.items():
-                            for halfTurn_nr, halfTurn in block.half_turns.items():
-                                insu = halfTurn.corners.insulated
-                                bare = halfTurn.corners.bare
-
-                                xInsuPos.append([insu.iL.x, insu.oL.x, insu.oR.x, insu.iR.x])
-                                yInsuPos.append([insu.iL.y, insu.oL.y, insu.oR.y, insu.iR.y])
-                                xBarePos.append([bare.iL.x, bare.oL.x, bare.oR.x, bare.iR.x])
-                                yBarePos.append([bare.iL.y, bare.oL.y, bare.oR.y, bare.iR.y])
-                                iConductor.append(block.current_sign)
-        return xInsuPos, yInsuPos, xBarePos, yBarePos, iConductor
