@@ -342,11 +342,23 @@ class Pole(BaseModel):
     bore_center: Coord = Coord()
 
 
+class Order(BaseModel):
+    """
+        Class for electrical order (block location)
+    """
+    coil: int = None
+    pole: int = None
+    layer: int = None
+    winding: int = None
+    block: int = None
+
+
 class CoilData(BaseModel):
     """
         Class for coils
     """
     coils: Dict[int, Pole] = {}
+    electrical_order: List[Order] = []
 
 
 class RoxieData(BaseModel):
