@@ -1,6 +1,6 @@
 def lookupModelDataToLEDET(key: str, mode='data2ledet'):
     """
-        Retrieves the correct LEDET parameter name for a DataModel input
+        Retrieves the correct LEDET parameter name for a DataModelMagnet input
         The argument mode is used to define the direction, either "data2ledet" or "ledet2data"
     """
     lookup = {
@@ -12,7 +12,7 @@ def lookupModelDataToLEDET(key: str, mode='data2ledet'):
         'CoilWindings.conductor_to_group': 'type_to_group',  # TODO: change name, or make it obsolete
         # Note that nT is NOT defined here (at the moment it is read from map2d file)
         'CoilWindings.group_to_coil_section': 'GroupToCoilSection',
-        'CoilWindings.polarities_in_group': 'polarities_inGroup',
+        'CoilWindings.polarities_in_group': 'polarities_inGroup',  # TODO Consider removing if removed from DataModelMagnet
         'CoilWindings.half_turn_length': 'l_mag_inGroup',
         'CoilWindings.electrical_pairs.group_together': 'elPairs_GroupTogether',
         'CoilWindings.electrical_pairs.reversed': 'elPairs_RevElOrder',
@@ -170,6 +170,7 @@ def lookupModelDataToLEDET(key: str, mode='data2ledet'):
         'Options_LEDET.field_map_files.columnsXY': 'columnsXY',
         'Options_LEDET.field_map_files.columnsBxBy': 'columnsBxBy',
         'Options_LEDET.field_map_files.flagPlotMTF': 'flagPlotMTF',
+        'Options_LEDET.field_map_files.fieldMapNumber': 'fieldMapNumber',
 
         'Options_LEDET.input_generation_options.flag_typeWindings': 'flag_typeWindings',
         'Options_LEDET.input_generation_options.flag_calculateInductanceMatrix': 'flag_calculateInductanceMatrix',
@@ -280,7 +281,7 @@ def lookupModelDataToLEDET(key: str, mode='data2ledet'):
 
 def lookupWindings(key: int):
     """
-        Retrieves the correct LEDET parameter name for a DataModel input
+        Retrieves the correct LEDET parameter name for a DataModelMagnet input
     """
     typeWindings = {
         0: 'multipole',
@@ -295,7 +296,7 @@ def lookupSuperconductor(key: str):
     """
     superconductor = {
         "Nb-Ti": 1, "NbTi": 1,
-        "Nb3Sn (Summer's fit)": 2, "Nb3Sn": 2,  # TODO: Note that this means that the generic entry "Nb3Sn" in the DataModel file defaults to Summer's fir in LEDET
+        "Nb3Sn (Summer's fit)": 2, "Nb3Sn": 2,  # TODO: Note that this means that the generic entry "Nb3Sn" in the DataModelMagnet file defaults to Summer's fir in LEDET
         "BSCCO2212": 3,
         "Nb3Sn(Bordini's fit)": 4
     }
