@@ -15,11 +15,11 @@ class DriverLEDET:
             print('path_exe =          {}'.format(path_exe))
             print('path_folder_LEDET = {}'.format(path_folder_LEDET))
 
-    def run_LEDET(self, nameMagnet: str, simsToRun: str):
+    def run_LEDET(self, nameMagnet: str, simsToRun: str, simFileType: str = '.xlsx'):
         '''
         ** Run LEDET model **
         :param nameMagnet: Name of the magnet model to run
-        :param simsToRun: Number of the simulation to run
+        :param simsToRun: Numbers identifyinh the simulations to run
         :return:
         '''
         # Unpack arguments
@@ -32,9 +32,10 @@ class DriverLEDET:
             print('path_folder_LEDET = {}'.format(path_folder_LEDET))
             print('nameMagnet =        {}'.format(nameMagnet))
             print('simsToRun =         {}'.format(simsToRun))
+            print('simFileType =       {}'.format(simFileType))
 
         # Run model
-        subprocess.call([path_exe, path_folder_LEDET, nameMagnet, simsToRun])
+        subprocess.call([path_exe, path_folder_LEDET, nameMagnet, simsToRun, simFileType])
 
 
 
