@@ -155,13 +155,8 @@ class ParserLEDET:
         template_LEDET_variables_sheet = get_template_LEDET_variables_sheet()
 
         # Define optional variables, which will be written only if present in the dataclass
-        optional_variables_input_sheet = [
-            'alpha_Nb3Sn0_inGroup',
-            'f_scaling_Jc_BSCCO2212_inGroup',
-            'overwrite_f_internalVoids_inGroup', 'overwrite_f_externalVoids_inGroup',
-            'f_RRR1_Cu_inGroup', 'f_RRR2_Cu_inGroup', 'f_RRR3_Cu_inGroup',
-            'RRR1_Cu_inGroup', 'RRR2_Cu_inGroup', 'RRR3_Cu_inGroup',
-            ]
+        optional_variables_input_sheet  = _getOptionalVariables_input()
+        optional_variables_output_sheet = _getOptionalVariables_output()
 
         ### Inputs sheet
         name_sheet_inputs = 'Inputs'  # This defines the name of the sheet and also of the variable group
@@ -180,13 +175,13 @@ class ParserLEDET:
             # Skip optional variables if they have 0 elements
             if name in optional_variables_input_sheet:
                 if isinstance(value, list) and len(value) == 0:
-                    print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
+                    if verbose: print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
                     continue
                 elif isinstance(value, np.ndarray) and value.shape[0] == 0:
-                    print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
+                    if verbose: print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
                     continue
-                elif len(value) == 1 and value is None:
-                    print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
+                elif value is None:
+                    if verbose: print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
                     continue
 
             # Assign value to the variable sheet
@@ -203,8 +198,22 @@ class ParserLEDET:
                 LEDET_options_sheet.append(row)  # Leave the row unchanged
                 continue  # stop treating this row: go to the next row
 
-            # Assign value to the variable sheet
+            # Get value of the current parameter
             value = self.builder_ledet.getAttribute(name_sheet_options, name)
+
+            # Skip optional variables if they have 0 elements
+            if name in optional_variables_output_sheet:
+                if isinstance(value, list) and len(value) == 0:
+                    if verbose: print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
+                    continue
+                elif isinstance(value, np.ndarray) and value.shape[0] == 0:
+                    if verbose: print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
+                    continue
+                elif value is None:
+                    if verbose: print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
+                    continue
+
+            # Assign value to the variable sheet
             LEDET_options_sheet.append([name, value, description])
 
         ### Plots sheet
@@ -314,7 +323,7 @@ class ParserLEDET:
             print('New file ' + nameFileLEDET + ' generated.')
         return
 
-    def writeLedetWorkbookSheet(self, book, sheet, variableGroup, variableLabels):
+    def writeLedetWorkbookSheet(self, book, sheet, variableGroup, variableLabels, verbose: bool = False):
         """
             **Write one sheet of a LEDET input file**
 
@@ -375,10 +384,10 @@ class ParserLEDET:
             # Skip optional variables if they have 0 elements
             if attribute in optional_variables:
                 if isinstance(builder_ledet.getAttribute(variableGroup, attribute), list) and len(builder_ledet.getAttribute(variableGroup, attribute)) == 0:
-                    print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(attribute))
+                    if verbose: print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(attribute))
                     continue
                 elif isinstance(builder_ledet.getAttribute(variableGroup, attribute), np.ndarray) and builder_ledet.getAttribute(variableGroup, attribute).shape[0] == 0:
-                    print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(attribute))
+                    if verbose: print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(attribute))
                     continue
 
             # Skip writing conductor resistance fraction if not enabled
@@ -485,6 +494,10 @@ class ParserLEDET:
         else:
             print("Skipped consistency checks.")
 
+        # Define optional variables, which will be written only if present in the dataclass
+        optional_variables_input_sheet = _getOptionalVariables_input()
+        optional_variables_output_sheet = _getOptionalVariables_output()
+
         # Write LEDET data into a dictionary
         ledet_data_dict = {
             **builder_ledet.Inputs.__dict__,
@@ -492,9 +505,22 @@ class ParserLEDET:
             **builder_ledet.Plots.__dict__,
             **builder_ledet.Variables.__dict__}  # to add program-specific variables
 
+        # Skip optional variables if they have 0 elements
+        for name, value in ledet_data_dict.copy().items():
+            if (name in optional_variables_input_sheet) or (name in optional_variables_output_sheet):
+                if isinstance(value, list) and len(value) == 0:
+                    if verbose: print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
+                    del ledet_data_dict[name]
+                elif isinstance(value, np.ndarray) and value.shape[0] == 0:
+                    if verbose: print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
+                    del ledet_data_dict[name]
+                elif value is None:
+                    if verbose: print('Variable {} is optional and has 0 elements, hence it will be skipped.'.format(name))
+                    del ledet_data_dict[name]
+
         # Write output .json file
         with open(full_path_file_name, 'w') as outfile:
-            json.dump(ledet_data_dict, outfile, cls=NumpyEncoder)
+            json.dump(ledet_data_dict, outfile, cls=NumpyEncoder, indent=4)
 
         # Display time stamp
         currentDT = datetime.datetime.now()
@@ -839,6 +865,23 @@ class ParserLEDET:
 
 
 #######################  Helper functions - START  #######################
+def _getOptionalVariables_input():
+    # Define optional variables, which will be written only if present in the dataclass
+    optional_variables_input_sheet = [
+        'alpha_Nb3Sn0_inGroup',
+        'f_scaling_Jc_BSCCO2212_inGroup',
+        'overwrite_f_internalVoids_inGroup', 'overwrite_f_externalVoids_inGroup',
+        'f_RRR1_Cu_inGroup', 'f_RRR2_Cu_inGroup', 'f_RRR3_Cu_inGroup',
+        'RRR1_Cu_inGroup', 'RRR2_Cu_inGroup', 'RRR3_Cu_inGroup',
+        ]
+    return optional_variables_input_sheet
+
+def _getOptionalVariables_output():
+    optional_variables_output_sheet = [
+        'fieldMapNumber',
+        ]
+    return optional_variables_output_sheet
+
 def CompareLEDETParameters(fileA, fileB, max_relative_error=1E-5, show_indices=0):
     '''
         Compare all the variables imported from two LEDET Excel input files
@@ -917,7 +960,7 @@ def copy_map2d(magnet_name: str, map2d_file_name: str, output_path: str, flagIro
 
 
 def copy_modified_map2d_ribbon_cable(magnet_name: str, map2d_file_name: str, output_path: str, geometry_ribbon_cable,
-                                     flagIron: int, flagSelfField: int, suffix: str = '', verbose: bool = False):
+                                     flagIron: int, flagSelfField: int, list_flag_ribbon: list, suffix: str = '', verbose: bool = False):
     ''' #TODO '''
 
     ### Check if output path exists
@@ -945,7 +988,7 @@ def copy_modified_map2d_ribbon_cable(magnet_name: str, map2d_file_name: str, out
     content_ROXIE_file = open(map2d_file_name, "r").read().split("\n")
 
     # Parse to ParserMap2d
-    NewMap2d = ParserMap2d.modify_map2d_ribbon_cable(map2d_file_name, geometry_ribbon_cable)
+    NewMap2d = ParserMap2d.modify_map2d_ribbon_cable(map2d_file_name, geometry_ribbon_cable, list_flag_ribbon)
     # Multiply 3rd,4th and 7th column to not have SI-Units
     ParserMap2d.multiply_column_by_value(NewMap2d, 3, 1000)
     ParserMap2d.multiply_column_by_value(NewMap2d, 4, 1000)
