@@ -1,10 +1,13 @@
+import os
 import datetime
+import ntpath
+import shutil
 import textwrap
+from pathlib import Path
 
 import yaml
 
-from steam_sdk.data.DataCircuit import DataCircuit, Component
-from steam_sdk.parsers.ParserExcel import compare_two_parameters
+from steam_sdk.data.DataModelCircuit import DataModelCircuit, Component
 
 
 class ParserPSPICE:
@@ -12,25 +15,28 @@ class ParserPSPICE:
         Class with methods to read/write PSPICE information from/to other programs
     """
 
-    def __init__(self, circuit_data: DataCircuit):
+    def __init__(self, circuit_data: DataModelCircuit, path_input: Path = None, output_path: str = ''):
         """
-            Initialization using a DataCircuit object containing circuit netlist structure
+            Initialization using a DataModelCircuit object containing circuit netlist structure
         """
 
-        self.circuit_data: DataCircuit = circuit_data
+        self.circuit_data: DataModelCircuit = circuit_data
+        self.path_input: str = path_input
+        self.output_path: str = output_path
 
 
-    def read_netlist(self, full_path_file_name: str, verbose: bool = False):
+    def read_netlist(self, full_path_file_name: str, flag_acquire_auxiliary_files: bool = False, verbose: bool = False):
         '''
         ** Reads a PSPICE netlist file **
 
         :param full_path_file_name:
+        :param flag_acquire_auxiliary_files: If True, add list of additional files to circuit_data
         :param verbose:
         :return: ModelCircuit dataclass with filled keys
         '''
 
         # Initialize
-        self.circuit_data = DataCircuit()
+        self.circuit_data = DataModelCircuit()
         # Set flags indicating that the last read line corresponds to an item that might span to the next line
         self._set_all_flags_to_false()
 
@@ -38,8 +44,9 @@ class ParserPSPICE:
             for row, line in enumerate(file):
                 if verbose: print(line.rstrip())
 
-                # Reset all flags to False if the line does not contain '+ '
-                if not '+ ' in line:
+                # Reset all flags to False if the line does not contain '+' in first position (excluding whitespaces)
+                # Note: When one of these flags is set to True it indicates that this line might contain additional parameters from the previous line
+                if (not '+' in line) and (not line.strip(' ')[0] == '+'):
                     self._set_all_flags_to_false()
 
                 # If the line is a comment, skip to the next
@@ -52,6 +59,10 @@ class ParserPSPICE:
                 if '.END' in line:
                     continue
 
+                # Remove spaces before and after the "=" equal sign to make the parser more robust
+                line = line.replace(' =', '=')
+                line = line.replace('= ', '=')
+
                 # Read stimuli
                 if '.STMLIB' in line:
                     line_split = line.rstrip('\n').split(' ')  # Note: .rstrip('\n') removes the endline. .spolit(' ') makes a list of space-divided strings
@@ -69,37 +80,40 @@ class ParserPSPICE:
                 # Read global parameters
                 if '.PARAM' in line:  # it also covers the case where ".PARAMS" is written
                     self.flag_read_global_parameters = True
+                    self.circuit_data.GlobalParameters.global_parameters = {}
                     continue
                 if self.flag_read_global_parameters and '+ ' in line:
                     line_split = line.rstrip('\n').split(' ')
                     line_split = line_split[1].split('=')
                     name  = line_split[0].strip('"')
                     value = line_split[1].strip('{').strip('}')
-                    self.circuit_data.GlobalParameters.global_parameters.append([str(name), str(value)])
+                    self.circuit_data.GlobalParameters.global_parameters[str(name)] = str(value)
                     continue
 
                 # Read options
                 if '.OPTION' in line:  # it also covers the case where ".OPTIONS" is written
                     self.flag_read_options = True
+                    self.circuit_data.Options.options_simulation = {}
                     continue
                 if self.flag_read_options and '+ ' in line:
-                    line_split = line.rstrip('\n').split(' ')
+                    line_split = line.rstrip('\n').strip(' ').split(' ')
                     line_split = line_split[1].split('=')
                     name  = line_split[0].strip('"')
                     value = line_split[1].strip('{').strip('}')
-                    self.circuit_data.Options.options_simulation.append([str(name), str(value)])
+                    self.circuit_data.Options.options_simulation[str(name)] = str(value)
                     continue
 
                 # Read options
                 if '.AUTOCONVERGE' in line:
                     self.flag_read_autoconverge = True
+                    self.circuit_data.Options.options_autoconverge = {}
                     continue
                 if self.flag_read_autoconverge and '+ ' in line:
-                    line_split = line.rstrip('\n').split(' ')
+                    line_split = line.rstrip('\n').strip(' ').split(' ')
                     line_split = line_split[1].split('=')
                     name  = line_split[0].strip('"')
                     value = line_split[1].strip('{').strip('}')
-                    self.circuit_data.Options.options_autoconverge.append([str(name), str(value)])
+                    self.circuit_data.Options.options_autoconverge[str(name)] = str(value)
                     continue
 
                 # Read analysis
@@ -121,6 +135,7 @@ class ParserPSPICE:
                 # Read time schedule
                 if '+ {SCHEDULE(' in line:
                     self.flag_read_time_schedule = True
+                    self.circuit_data.Analysis.simulation_time.time_schedule = {}
                     continue
                 # If the line is the end of the time schedule section, skip to the next
                 if '+ )}' in line or '+)}' in line:
@@ -130,7 +145,7 @@ class ParserPSPICE:
                     line_split = line_split[1].split(',')
                     name  = line_split[0].strip(' ').strip('\t').strip(' ')
                     value = line_split[1].strip(' ').strip('\t').strip(' ')
-                    self.circuit_data.Analysis.simulation_time.time_schedule.append([str(name), str(value)])
+                    self.circuit_data.Analysis.simulation_time.time_schedule[str(name)] = str(value)
                     continue
 
                 # Read probe
@@ -155,46 +170,120 @@ class ParserPSPICE:
                     self.circuit_data.AuxiliaryFiles.files_to_include.append(str(value))
                     continue
 
-                # Read netlist - If this part of the code is reached without hitting "continue", it means that this line does not define a special command, and hence it defines a component of the netlist
-                self.flag_read_parametrized_component = True
-                if '+ PARAM' in line:  # This line defines parameters of the component
+                # Read netlist - If this part of the code is reached without hitting "continue", it means that this
+                # line does not define a special command, and hence it defines a component of the netlist
+                if (not self.circuit_data.Netlist[0].name == None) and (not self.flag_read_parametrized_component):  # Append a new component (unless it is the first component)
+                    self.circuit_data.Netlist.append(Component())
+
+                # Special case: Stimulus-controlled component
+                if 'STIMULUS=' in line.replace(' ', ''):  # this covers the cases "STIMULUS = ", "STIMULUS=", etc
+                    line_split = line.rstrip('\n').split('(')  # TODO: Improve the logic, which now fails if the value includes the character "("
+                    name = line_split[0].strip(' ')
+                    line_split = line_split[1].rstrip('\n').split(')')
+                    nodes = line_split[0].split(' ')
+                    line_split = line_split[1].split('=')
+                    value = line_split[1].strip(' { } ')  # Strip spaces and bracket at the extremities
+                    self.circuit_data.Netlist[-1].type  = 'stimulus-controlled component'
+                    self.circuit_data.Netlist[-1].name  = str(name)
+                    self.circuit_data.Netlist[-1].nodes = nodes
+                    self.circuit_data.Netlist[-1].value = str(value)
+                    continue
+
+                # Special case: Controlled-source component
+                # E: Voltage-controlled voltage source
+                # F: Current-controlled current source
+                # G: Voltage-controlled current source
+                # H: Current-controlled voltage source
+                if ('VALUE ' in line) and ((line[0] == 'E') or (line[0] == 'F') or (line[0] == 'G') or (line[0] == 'H')):
+                    line_before, _, line_after = line.rstrip('\n').partition('(')  # Take the part of the string before and after the first "(" char
+                    name = line_before.strip(' ')
+                    line_before, _, line_after = line_after.rstrip(' ').partition(')')  # Take the part of the string before and after the first ")" char
+                    nodes = line_before.split(' ')
+                    line_after = line_after.partition('VALUE ')[2]  # Take the part of the string after the first "VALUE " chars
+                    value = line_after.strip(' { } ')  # Strip spaces and bracket at the extremities
+                    self.circuit_data.Netlist[-1].type  = 'controlled-source component'
+                    self.circuit_data.Netlist[-1].name  = str(name)
+                    self.circuit_data.Netlist[-1].nodes = nodes
+                    self.circuit_data.Netlist[-1].value = str(value)
+                    continue
+
+                # Special case: Parametrized component
+                if ('x' in line and line[0] == 'x') or ('X' in line and line[0] == 'X'):
+                    self.flag_read_parametrized_component = True
+                    line_before, _, line_after = line.rstrip('\n').partition('(')  # Take the part of the string before and after the first "(" char
+                    name = line_before.strip(' ')
+                    line_before, _, line_after = line_after.rstrip(' ').partition(')')  # Take the part of the string before and after the first ")" char
+                    nodes = line_before.split(' ')
+                    value = line_after.strip(' { } ')  # Strip spaces and bracket at the extremities
+                    self.circuit_data.Netlist[-1].type  = 'parametrized component'
+                    self.circuit_data.Netlist[-1].name  = str(name)
+                    self.circuit_data.Netlist[-1].nodes = nodes
+                    self.circuit_data.Netlist[-1].value = str(value)
+                    continue
+
+                # Special case: Parameters of a parametrized component
+                if self.flag_read_parametrized_component and '+ PARAM' in line:  # This line defines parameters of the component
                     # Reminder: This type of line looks like this: + PARAMS: name1={value1} name2={value2} name3={value3}
                     line_split = line.rstrip('\n').partition(':')[2]  # Take the part of the string after the first "+" char
                     # line_split = line.rstrip('\n').split(':')  # Split line in two, before and after the "+ PARAMS:" command
                     # line_split = line_split[1].strip(' ')      # Take the second element and remove whitespaces at its extremities
-                    line_split = line_split.split('}')         # Split into different parameters
+                    line_split = line_split.split(' ')         # Split into different parameters
+                    self.circuit_data.Netlist[-1].parameters = {}
                     for par in line_split:                     # Loop through the parameters
                         par_split = par.split('=')             # Split into name and value of the parameter
                         if len(par_split) == 2:                # Only take into account entries with two elements (for example, avoid None)
                             name_par = str(par_split[0].strip(' '))
-                            value_par = str(par_split[1].strip(' { '))  # Strip spaces and bracket at the extremities
-                            self.circuit_data.Netlist[-1].parameters.append([str(name_par), str(value_par)])
+                            value_par = str(par_split[1].strip(' { } '))  # Strip spaces and bracket at the extremities
+                            # self.circuit_data.Netlist[-1].parameters.append([str(name_par), str(value_par)])
+                            self.circuit_data.Netlist[-1].parameters[str(name_par)] = str(value_par)
                     continue
-                elif '+' in line and line[0] == '+':  # This line defines additional parameters of the component
+                elif self.flag_read_parametrized_component and '+' in line and line[0] == '+':  # This line defines additional parameters of the component
                     # Reminder: This type of line looks like this: + PARAMS: name1={value1} name2={value2} name3={value3}
                     line_split = line.rstrip('\n').partition('+')[2]        # Take the part of the string after the first "+" char
-                    line_split = line_split.split('}')                      # Split into different parameters
+                    line_split = line_split.split(' ')                      # Split into different parameters
                     for par in line_split:                                  # Loop through the parameters
                         par_split = par.split('=')                          # Split into name and value of the parameter
                         if len(par_split) == 2:                             # Only take into account entries with two elements (for example, avoid None)
                             name_par  = str(par_split[0].strip(' '))
-                            value_par = str(par_split[1].strip(' { '))  # Strip spaces and bracket at the extremities
-                            self.circuit_data.Netlist[-1].parameters.append([str(name_par), str(value_par)])
+                            value_par = str(par_split[1].strip(' { } '))  # Strip spaces and bracket at the extremities
+                            # self.circuit_data.Netlist[-1].parameters.append([str(name_par), str(value_par)])
+                            self.circuit_data.Netlist[-1].parameters[str(name_par)] = str(value_par)
                     continue
-                # If this part of the code is reached, the line defines a new component
 
-                line_split = line.rstrip('\n').split('(')  # TODO: Improve the logic, which now fails if the value includes the character "("
-                name = line_split[0].strip(' ')
-                line_split = line_split[1].rstrip('\n').split(')')
-                nodes = line_split[0].split(' ')
-                value = line_split[1].strip(' { } ')  # Strip spaces and bracket at the extremities
-                if not self.circuit_data.Netlist[0].name == None:  # do not append a new element if it is the first component
-                    self.circuit_data.Netlist.append(Component())
-                self.circuit_data.Netlist[-1].type = 'component'
+                # If this part of the code is reached, the line defines standard component
+                line_before, _, line_after = line.rstrip('\n').partition('(')  # Take the part of the string before and after the first "(" char
+                name = line_before.strip(' ')
+                line_before, _, line_after = line_after.rstrip(' ').partition(')')  # Take the part of the string before and after the first ")" char
+                nodes = line_before.split(' ')
+                value = line_after.strip(' { } ')  # Strip spaces and bracket at the extremities
+                self.circuit_data.Netlist[-1].type  = 'standard component'
                 self.circuit_data.Netlist[-1].name  = str(name)
                 self.circuit_data.Netlist[-1].nodes = nodes
                 self.circuit_data.Netlist[-1].value = str(value)
-                # self.circuit_data.Netlist[-1].type  =  # TODO: Add logic
+
+        # Simplify options keys if they have default values
+        if self.circuit_data.Options.options_simulation == get_default_options():
+            print('Default simulation options are applied.')
+            self.circuit_data.Options.options_simulation = {'all options': 'default'}
+        if self.circuit_data.Options.options_autoconverge == get_default_autoconverge_options():
+            print('Default simulation autoconvergence options are applied.')
+            self.circuit_data.Options.options_autoconverge = {'all options': 'default'}
+
+        if flag_acquire_auxiliary_files:
+            # Read entries of required additional files and add them to the yaml dictionary
+            # Add stimulus files (defined with ".STMLIB" option)
+            for file in self.circuit_data.Stimuli.stimulus_files:
+                self.circuit_data.GeneralParameters.additional_files.append(file)
+            # Add component library files (defined with ".LIB" option)
+            for file in self.circuit_data.Libraries.component_libraries:
+                self.circuit_data.GeneralParameters.additional_files.append(file)
+            # Add files to include (defined with ".INC" option)
+            for file in self.circuit_data.AuxiliaryFiles.files_to_include:
+                self.circuit_data.GeneralParameters.additional_files.append(file)
+            if verbose:
+                print('Option flag_acquire_auxiliary_files set to {}'.format(flag_acquire_auxiliary_files))
+                for file in self.circuit_data.GeneralParameters.additional_files:
+                    print('File {} added to the list of files to add to the model.'.format(file))
 
         return self.circuit_data
 
@@ -239,17 +328,13 @@ class ParserPSPICE:
 
         # Prepare global parameters
         rows_global_parameters = []
-        for s, global_parameter in enumerate(self.circuit_data.GlobalParameters.global_parameters):
-            # Check input size
-            if len(global_parameter) != 2:
-                raise Exception ('All global parameters entries must have 2 elements (name, value), but parameter {}, in position #{} has {} elements.'.format(global_parameter, s+1, len(global_parameter)))
+        if self.circuit_data.GlobalParameters.global_parameters:
             # Add comment and .PARAM command before the first entry
-            if s == 0:
-                rows_global_parameters.append(add_comment('**** Global parameters ****'))  # Add header of this section
-                rows_global_parameters.append('.PARAM')  # Add header of this section
-            # Add global parameter entry
-            name, value = global_parameter[0], global_parameter[1]
-            rows_global_parameters.append(add_global_parameter(name, value))
+            rows_global_parameters.append(add_comment('**** Global parameters ****'))  # Add header of this section
+            rows_global_parameters.append('.PARAM')  # Add header of this section
+            # Add global parameter entries
+            for name, value in self.circuit_data.GlobalParameters.global_parameters.items():
+                rows_global_parameters.append(add_global_parameter(name, value))
 
         # Prepare netlist
         rows_netlist = []
@@ -257,88 +342,70 @@ class ParserPSPICE:
         if not self.circuit_data.Netlist[0].type:
             raise Exception('At least one netlist entry of known type must be added. Supported component types:\n' +
                             '- comment\n' +
-                            '- component\n'
+                            '- standard component\n'
+                            '- stimulus-controlled component\n'
+                            '- controlled-source component\n'
+                            '- parametrized component\n'
                             'Netlist cannot be generated.')
         rows_global_parameters.append(add_comment('**** Netlist ****'))  # Add header of this section
         for s, component in enumerate(self.circuit_data.Netlist):
             # Read keys
-            name, nodes, value, model, parameters, type = \
-                component.name, component.nodes, component.value, component.model, component.parameters, component.type
+            name, nodes, value, parameters, type = \
+                component.name, component.nodes, component.value, component.parameters, component.type
 
             # Add the relevant row depending on the component type
             if type == 'comment':
                 if verbose: print('Netlist entry {} in position #{} is treated as a comment.'.format(name, s + 1))
                 rows_netlist.append(add_comment(value))
-            elif type == 'component':
+            elif type == 'standard component':
                 if name == None or nodes == None or value == None:
                     raise Exception('Netlist component in position #{} is of type {} and requires name, nodes, and value.'.format(s+1, type))
-                if parameters == None:
-                    if verbose: print('Netlist entry {} in position #{} is treated as a non-parametrized component.'.format(name, s + 1))
-                    rows_netlist.append(add_standard_component(name, nodes, value))
-                else:
-                    if verbose: print('Netlist entry {} in position #{} is treated as a parametrized component.'.format(name, s+1))
-                    rows_netlist.append(add_parametrized_component(name, nodes, value, parameters))
+                if verbose: print('Netlist entry {} in position #{} is treated as a standard component.'.format(name, s + 1))
+                rows_netlist.append(add_standard_component(name, nodes, value))
+            elif type == 'stimulus-controlled component':
+                if name == None or nodes == None or value == None:
+                    raise Exception('Netlist component in position #{} is of type {} and requires name, nodes, and value.'.format(s+1, type))
+                if verbose: print('Netlist entry {} in position #{} is treated as a stimulus-controlled component.'.format(name, s + 1))
+                rows_netlist.append(add_stimulus_controlled_component(name, nodes, value))
+            elif type == 'controlled-source component':
+                if name == None or nodes == None or value == None:
+                    raise Exception('Netlist component in position #{} is of type {} and requires name, nodes, and value.'.format(s+1, type))
+                if verbose: print('Netlist entry {} in position #{} is treated as a controlled-source component.'.format(name, s + 1))
+                rows_netlist.append(add_controlled_source_component(name, nodes, value))
+            elif type == 'parametrized component':
+                if name == None or nodes == None or value == None:
+                    raise Exception('Netlist component in position #{} is of type {} and requires name, nodes, and value.'.format(s+1, type))
+                if verbose: print('Netlist entry {} in position #{} is treated as a parametrized component.'.format(name, s + 1))
+                rows_netlist.append(add_parametrized_component(name, nodes, value, parameters))
             else:
                 raise Exception ('Netlist entry {} in position #{} has an unknown type: {}.'.format(component.name, s+1, type))
 
         # Prepare options - Simulation options
         rows_options = []
-
         options = self.circuit_data.Options.options_simulation
-        if options == ['default']:
+        if options == {'all options': 'default'}:
             print('Default simulation options are applied.')
-            options = [
-                ['RELTOL', '0.0001'],
-                ['VNTOL', '0.00001'],
-                ['ABSTOL', '0.0001'],
-                ['CHGTOL', '0.000000000000001'],
-                ['GMIN', '0.000000000001'],
-                ['ITL1', '150'],
-                ['ITL2', '20'],
-                ['ITL4', '10'],
-                ['TNOM', '27'],
-                ['NUMDGT', '8'],
-            ]
-        for s, option in enumerate(options):
-            # Check input size
-            if len(option) != 2:
-                raise Exception(
-                    'All global parameters entries must have 2 elements (name, value), but parameter {}, in position #{} has {} elements.'.format(
-                        option, s + 1, len(option)))
+            options = get_default_options()
+        if options:
             # Add comment and .OPTIONS command before the first entry
-            if s == 0:
-                rows_options.append(
-                    add_comment('**** Simulation options ****'))  # Add header of this section
-                rows_options.append('.OPTIONS')  # Add header of this section
-            # Add global parameter entry
-            name, value = option[0], option[1]
-            rows_options.append(add_option(name, value))
+            rows_options.append(add_comment('**** Simulation parameters ****'))  # Add header of this section
+            rows_options.append('.OPTIONS')  # Add header of this section
+            # Add option entries
+            for name, value in options.items():
+                rows_options.append(add_option(name, value))
 
         # Prepare options - Autoconverge simulation options
         options_autoconverge = self.circuit_data.Options.options_autoconverge
-        if options_autoconverge == ['default']:
+        if options_autoconverge == {'all options': 'default'}:
             print('Default autoconverge simulation options_autoconverge are applied.')
-            options_autoconverge = [
-                ['RELTOL', '0.05'],
-                ['VNTOL', '0.0001'],
-                ['ABSTOL', '0.0001'],
-                ['ITL1', '1000'],
-                ['ITL2', '1000'],
-                ['ITL4', '1000'],
-                ['PIVTOL', '0.0000000001'],
-                ]
-        for s, option in enumerate(options_autoconverge):
-            # Check input size
-            if len(option) != 2:
-                raise Exception(
-                    'All global parameters entries must have 2 elements (name, value), but parameter {}, in position #{} has {} elements.'.format(option, s + 1, len(option)))
+            options_autoconverge = get_default_autoconverge_options()
+        if options_autoconverge:
             # Add comment and .AUTOCONVERGE command before the first entry
-            if s == 0:
-                rows_options.append(add_comment('**** Simulation autoconverge options ****'))  # Add header of this section
-                rows_options.append('.AUTOCONVERGE')  # Add header of this section
-            # Add global parameter entry
-            name, value = option[0], option[1]
-            rows_options.append(add_option(name, value))
+            rows_options.append(add_comment('**** Simulation autoconverge options ****'))  # Add header of this section
+            rows_options.append('.AUTOCONVERGE')  # Add header of this section
+            # Add option entries
+            for name, value in options_autoconverge.items():
+                rows_options.append(add_option(name, value))
 
         # Prepare analysis settings
         rows_analysis = []
@@ -350,7 +417,7 @@ class ParserPSPICE:
             min_time_step = self.circuit_data.Analysis.simulation_time.min_time_step
             time_schedule = self.circuit_data.Analysis.simulation_time.time_schedule
             # Check inputs
-            if not time_start:
+            if time_start == None:
                 time_start = '0'
                 print('Parameter time_start set to {} by default.'.format(time_start))
             if not time_end:
@@ -360,7 +427,7 @@ class ParserPSPICE:
             # Add analysis entry
             rows_analysis.append(add_transient_analysis(time_start, time_end, min_time_step))
             # If defined, add time schedule (varying minimum time stepping) entry
-            if time_schedule and len(time_schedule) > 0 and len(time_schedule[0]):
+            if time_schedule and len(time_schedule) > 0:
                 rows_analysis.append(add_transient_time_schedule(time_schedule))
         elif analysis_type == 'frequency':
             # TODO: frequency analysis. EXAMPLE: .AC DEC 50 1Hz 200kHz
@@ -416,10 +483,10 @@ class ParserPSPICE:
 
 
     def readFromYaml(self, full_path_file_name: str, verbose: bool = False):
-        # Load yaml keys into DataCircuit dataclass
+        # Load yaml keys into DataModelCircuit dataclass
         with open(full_path_file_name, "r") as stream:
             dictionary_yaml = yaml.safe_load(stream)
-            self.circuit_data = DataCircuit(**dictionary_yaml)
+            self.circuit_data = DataModelCircuit(**dictionary_yaml)
         if verbose:
             print('File ' + full_path_file_name + ' read.')
 
@@ -439,6 +506,36 @@ class ParserPSPICE:
             print('New file ' + full_path_file_name + ' generated.')
 
 
+    def copy_additional_files(self, verbose: bool = False):
+        '''
+            Copy additional files
+        :return: Number of copied files
+        '''
+        for file_to_copy in self.circuit_data.GeneralParameters.additional_files:
+            if not os.path.isabs(file_to_copy) and self.path_input:
+                # If the provided path is relative, use the path_input as the root folder (if available)
+                file_to_copy_relative = os.path.join(self.path_input, file_to_copy)
+                file_to_copy = file_to_copy_relative
+                if verbose:
+                    print('Relative path changed from {} to {}.'.format(file_to_copy, file_to_copy_relative))
+
+            file_name = ntpath.basename(file_to_copy)
+            file_to_write = os.path.join(self.output_path, file_name)
+            if os.path.isfile(os.path.join(file_to_copy)):
+                print('File {} exists'.format(file_to_copy))
+            else:
+                print('File {} does not exist'.format(file_to_copy))
+            if os.path.isdir(self.output_path):
+                print('Dir {} exists'.format(self.output_path))
+            else:
+                print('Dir {} does not exist'.format(self.output_path))
+            shutil.copyfile(Path(file_to_copy), Path(file_to_write))
+
+            if verbose:
+                print('Additional file copied from {} to {}.'.format(file_to_copy, file_to_write))
+        return len(self.circuit_data.GeneralParameters.additional_files)
+
+
 #######################  Helper functions - START  #######################
 def add_comment(text: str):
     ''' Format comment row '''
@@ -468,28 +565,43 @@ def add_standard_component(name: str, nodes: list, value: str):
     formatted_text = name + ' (' + str_nodes + ') ' + '{' + value + '}'
     return formatted_text
 
-# def add_stimulus_controlled_component(name: str, nodes: list, value: str):
-#     ''' Format stimulus-controlled component netlist row '''
-#     str_nodes = " ".join(nodes)  # string with space-separated nodes
-#     str_stimulus = 'STIMULUS = ' + value
-#     formatted_text = name + ' (' + str_nodes + ') ' + str_stimulus
-#     return formatted_text
+def add_stimulus_controlled_component(name: str, nodes: list, value: str):
+    ''' Format stimulus-controlled component netlist row '''
+    str_nodes = " ".join(nodes)  # string with space-separated nodes
+    str_stimulus = 'STIMULUS = ' + value
+    formatted_text = name + ' (' + str_nodes + ') ' + str_stimulus
+    return formatted_text
 
-def add_parametrized_component(name: str, nodes: list, value: str, parameters: list):
+def add_controlled_source_component(name: str, nodes: list, value: str):
+    ''' Format controlled-source component netlist row '''
+    str_nodes = " ".join(nodes)  # string with space-separated nodes
+    str_stimulus = 'VALUE ' + '{' + value + '}'
+    formatted_text = name + ' (' + str_nodes + ') ' + str_stimulus
+    return formatted_text
+
+def add_parametrized_component(name: str, nodes: list, value: str, parameters: dict):
     ''' Format parametrized component netlist row '''
     str_nodes = " ".join(nodes)  # string with space-separated nodes
-    formatted_component = name + ' (' + str_nodes + ') ' + '{' + value + '}' + '\n'  # First row, which defines the component
-    formatted_parameters = '+ PARAMS:'  # First part of the string in the second row, which defines the component parameters
-    for parameter in parameters:
-        if len(parameter) != 2:
-            raise Exception ('All parameters entries in a parametrized element must have 2 elements (name, value), but parameter {} has {} elements.'.format(name, len(parameter)))
-        name_parameters, value_parameters = parameter[0], parameter[1]
-        formatted_parameters = formatted_parameters + ' ' + name_parameters + '={' + value_parameters + '}'
+    formatted_component = name + ' (' + str_nodes + ') ' + value  # First row, which defines the component
+    if parameters:
+        formatted_component = formatted_component + '\n'
+        formatted_parameters = '+ PARAMS:'  # First part of the string in the second row, which defines the component parameters
+        # for parameter in parameters:
+        #     if len(parameter) != 2:
+        #         raise Exception ('All parameters entries in a parametrized element must have 2 elements (name, value), but parameter {} has {} elements.'.format(name, len(parameter)))
+        #     name_parameters, value_parameters = parameter[0], parameter[1]
+        #     formatted_parameters = formatted_parameters + ' ' + name_parameters + '={' + value_parameters + '}'
+        for name_parameters, value_parameters in parameters.items():
+            # if len(parameter) != 2:
+            #     raise Exception ('All parameters entries in a parametrized element must have 2 elements (name, value), but parameter {} has {} elements.'.format(name, len(parameter)))
+            formatted_parameters = formatted_parameters + ' ' + name_parameters + '={' + value_parameters + '}'
 
-    # Make sure the maximum number of characters in each row does not exceed 132, which is the maximum that PSPICE supports
-    N_MAX_CHARS_PER_ROW = 130
-    formatted_parameters = textwrap.fill(formatted_parameters, N_MAX_CHARS_PER_ROW)
-    formatted_parameters = formatted_parameters.replace('\n', '\n+ ')  # Add "+ " at the beginning of each new line
+        # Make sure the maximum number of characters in each row does not exceed 132, which is the maximum that PSPICE supports
+        N_MAX_CHARS_PER_ROW = 130
+        formatted_parameters = textwrap.fill(formatted_parameters, N_MAX_CHARS_PER_ROW)
+        formatted_parameters = formatted_parameters.replace('\n', '\n+ ')  # Add "+ " at the beginning of each new line
+    else:
+        formatted_parameters = ''  # If no parameters are defined, empty string
 
     formatted_text = formatted_component + formatted_parameters  # Concatenate the two rows
     return formatted_text
@@ -499,23 +611,22 @@ def add_option(name: str, value: str):
     formatted_text = '+ ' + name + '=' + value
     return formatted_text
 
-def add_transient_analysis(time_start, time_end, min_time_step):
+def add_transient_analysis(time_start, time_end, min_time_step = None):
     ''' Format transient analysis row '''
     formatted_text = '.TRAN ' + str(time_start) + ' ' + str(time_end)
-    if min_time_step:
+    if not min_time_step == None:
         formatted_text = formatted_text + ' ' + str(min_time_step)
     return formatted_text
 
 def add_transient_time_schedule(time_schedule):
     ''' Format transient time schedule rows '''
     # If time_schedule is not defined, output will be None
-    if time_schedule == None or len(time_schedule) == 0 or len(time_schedule[0]) == 0:
+    if time_schedule == None or len(time_schedule) == 0:
         return None
 
     formatted_text = '+ {SCHEDULE(\n'
-    for t, time_entry in enumerate(time_schedule):
-        time_window_start, time_step_in_window = str(time_entry[0]), str(time_entry[1])
-        if t+1 == len(time_schedule):
+    for time_window_start, time_step_in_window in time_schedule.items():
+        if time_window_start == list(time_schedule.keys())[-1]:
             formatted_text = formatted_text + '+ ' + time_window_start + ', ' + time_step_in_window + '\n'  # the last entry must not have the comma
         else:
             formatted_text = formatted_text + '+ ' + time_window_start + ', ' + time_step_in_window + ',' + '\n'
@@ -527,7 +638,7 @@ def add_probe(probe_type: str, probe_variables: list):
     if probe_type == 'standard':
         formatted_text = '.PROBE'
     elif probe_type == 'CSDF':
-        formatted_text = '.PROBE /CSDF'
+        formatted_text = '.PROBE/CSDF'
     elif not probe_type:
         return None
     else:
@@ -551,6 +662,33 @@ def add_end_file():
     formatted_text = '.END'
     return formatted_text
 
+def get_default_options():
+    options = {
+        'RELTOL': '0.0001',
+        'VNTOL': '0.00001',
+        'ABSTOL': '0.0001',
+        'CHGTOL': '0.000000000000001',
+        'GMIN': '0.000000000001',
+        'ITL1': '150',
+        'ITL2': '20',
+        'ITL4': '10',
+        'TNOM': '27',
+        'NUMDGT': '8',
+    }
+    return options
+
+def get_default_autoconverge_options():
+    options_autoconverge = {
+        'RELTOL': '0.05',
+        'VNTOL': '0.0001',
+        'ABSTOL': '0.0001',
+        'ITL1': '1000',
+        'ITL2': '1000',
+        'ITL4': '1000',
+        'PIVTOL': '0.0000000001',
+    }
+    return options_autoconverge
+
 
 def ComparePSPICEParameters(fileA, fileB):
     '''
